@@ -18,7 +18,8 @@ from colander import (
     Email,
     Date,
     DateTime,
-    drop
+    Mapping,
+    drop,
 )
 
 from . import registry, TypeField, TypeFieldNode
@@ -28,7 +29,8 @@ from .json import JSONList
 __all__ = ['IntField', 'StringField', 'RangeField',
            'RegexField', 'EmailField', 'URLField',
            'EnumField', 'ChoicesField', 'DecimalField',
-           'DateField', 'DateTimeField', 'GroupField']
+           'DateField', 'DateTimeField', 'GroupField',
+           'AnnotationField']
 
 
 @registry.add('int')
@@ -49,6 +51,20 @@ class TextField(TypeField):
     hint = _('A text')
 
 
+@registry.add('annotation')
+class AnnotationField(TypeField):
+    required = False
+
+    @classmethod
+    def definition(cls, **kwargs):
+        schema = SchemaNode(Mapping(unknown="preserve"))
+
+        schema.add(SchemaNode(String(), name='label', missing=u''))
+        schema.add(SchemaNode(String(), name='type',
+                              validator=OneOf(["annotation"])))
+        return schema
+
+
 @registry.add('decimal')
 class DecimalField(TypeField):
     node = Decimal
@@ -217,7 +233,7 @@ class GroupField(TypeField):
     @classmethod
     def definition(cls, **kwargs):
         schema = super(GroupField, cls).definition(**kwargs)
-        # Keep ``required`` and ``type`` nodes only
+        # Keep the ``type`` node only
         schema.children = [c for c in schema.children
                            if c.name not in ('hint', 'name', 'required')]
         schema.add(SchemaNode(String(), name='description', missing=drop))
