@@ -177,7 +177,8 @@ class TodoModelTest(FunctionalTest, BaseWebTest):
                         "todo"
                     ],
                     "required": True,
-                    "label": "is it done or not"
+                    "label": "is it done or not",
+                    "display": "dropdown"
                 }
             ]
         }
@@ -193,6 +194,11 @@ class TodoModelTest(FunctionalTest, BaseWebTest):
     def update_record(self, entry):
         entry['status'] = 'done'
 
+    def test_definition_creation_does_keep_metadata_fields(self):
+        resp = self.app.get('/models/%s/definition' % self.model_id,
+                            headers=self.headers)
+        self.assertEquals(resp.json["fields"][1]["display"], "dropdown")
+
 
 class MushroomsModelTest(FunctionalTest, BaseWebTest):
 
@@ -266,3 +272,31 @@ class MushroomsModelTest(FunctionalTest, BaseWebTest):
         # after update
         self.assertCountEqual(feature['geometry']['coordinates'],
                               [[[0, 0], [0, 1], [1, 1], [0, 0]]])
+
+
+class AnnotationModelTest(BaseWebTest):
+
+    def test_annotation_attribute_can_be_provided(self):
+        resp = self.app.put_json('/models/annotation', {
+            'definition': {
+                "title": "annotation",
+                "description": "A list of my stuff to do",
+                "fields": [
+                    {
+                        "type": "annotation",
+                        "label": "The annotation item",
+                        "anotherField": "haha"
+                    }
+                ]
+            }
+        }, headers=self.headers)
+
+        resp = self.app.get('/models/annotation', headers=self.headers)
+        self.assertEquals(
+            resp.json['definition']['fields'][0],
+            {
+                u'label': u'The annotation item',
+                u'type': u'annotation',
+                u'anotherField': u'haha'
+            }
+        )
