@@ -1,10 +1,12 @@
+import copy
 import base64
 
 import mock
-from pyramid.security import Authenticated, Everyone
+from webtest.app import TestRequest
+import elasticsearch
 
-from daybed import __version__ as VERSION
-from daybed.permissions import PERMISSIONS_SET
+from daybed import __version__ as VERSION, API_VERSION
+from daybed.permissions import invert_permissions_matrix
 from daybed.backends.exceptions import (
     RecordNotFound, ModelNotFound
 )
@@ -17,6 +19,8 @@ MODEL_DEFINITION = {
         "title": "simple",
         "description": "One optional field",
         "fields": [{"name": "age",
+                    "hint": "An integer",
+                    "label": "Age",
                     "type": "int",
                     "required": False}]
     }
@@ -97,6 +101,27 @@ class HelloViewTest(BaseWebTest):
         headers['Authorization'] = 'Basic boom'
         self.app.get('/', headers=self.headers)
 
+    def test_redirect_to_version(self):
+        # We don't want the prefix to be automatically added for this test.
+        original_request_class = self.app.RequestClass
+
+        try:
+            self.app.RequestClass = TestRequest  # Standard RequestClass.
+
+            # GET on the hello view.
+            response = self.app.get('/')
+            self.assertEqual(response.status_int, 307)
+            self.assertEqual(response.location,
+                             'http://localhost/%s/' % API_VERSION)
+
+            # GET on the fields view.
+            response = self.app.get('/fields')
+            self.assertEqual(response.status_int, 307)
+            self.assertEqual(response.location,
+                             'http://localhost/%s/fields' % API_VERSION)
+        finally:
+            self.app.RequestClass = original_request_class
+
 
 class BasicAuthRegistrationTest(BaseWebTest):
     model_id = 'simple'
@@ -153,6 +178,64 @@ class SporeTest(BaseWebTest):
         self.assertEqual(resp.json['name'], 'daybed')
 
 
+class DefinitionViewsTest(BaseWebTest):
+
+    def setUp(self):
+        super(DefinitionViewsTest, self).setUp()
+        model = copy.deepcopy(MODEL_DEFINITION)
+        model['definition']['fields'].append({
+            "name": "name",
+            "type": "string"
+        })
+        model['permissions'] = {"system.Everyone": ["ALL"]}
+        model['records'] = [{'name': 'Snowden', 'age': 31}]
+        self.app.put_json('/models/test',
+                          model,
+                          headers=self.headers)
+
+    def test_definition_retrieval(self):
+        resp = self.app.get('/models/test/definition',
+                            headers=self.headers)
+        self.assertEqual(len(resp.json['fields']), 2)
+
+    def test_definition_update_returns_new_definition(self):
+        resp = self.app.put_json('/models/test/definition',
+                                 MODEL_DEFINITION['definition'],
+                                 headers=self.headers)
+        self.assertEqual(len(resp.json['fields']), 1)
+
+    def test_definition_update_must_be_valid(self):
+        definition = MODEL_DEFINITION['definition'].copy()
+        definition.pop('fields')
+        self.app.put_json('/models/test/definition',
+                          definition,
+                          headers=self.headers, status=400)
+
+    def test_definition_update_preserves_records(self):
+        self.app.put_json('/models/test/definition',
+                          MODEL_DEFINITION['definition'],
+                          headers=self.headers)
+        resp = self.app.get('/models/test/records',
+                            headers=self.headers)
+        self.assertEqual(len(resp.json['records']), 1)
+
+    def test_definition_update_preserves_permissions(self):
+        self.app.put_json('/models/test/definition',
+                          MODEL_DEFINITION['definition'],
+                          headers=self.headers)
+        resp = self.app.get('/models/test/permissions',
+                            headers=self.headers)
+        self.assertIn('system.Everyone', resp.json)
+
+    def test_model_creation_via_definition(self):
+        self.app.put_json('/models/new/definition',
+                          MODEL_DEFINITION['definition'],
+                          headers=self.headers)
+        resp = self.app.get('/models/new/records',
+                            headers=self.headers)
+        self.assertEqual(len(resp.json['records']), 0)
+
+
 class ModelsViewsTest(BaseWebTest):
 
     def __init__(self, *args, **kwargs):
@@ -202,26 +285,6 @@ class ModelsViewsTest(BaseWebTest):
         self.assertDictEqual(resp.json['permissions'],
                              {self.credentials['id']: MODEL_PERMISSIONS})
 
-    def test_post_model_definition_without_definition(self):
-        self.app.post_json('/models', {}, headers=self.headers, status=400)
-
-    def test_post_model_definition_without_records(self):
-        resp = self.app.post_json('/models',
-                                  MODEL_DEFINITION,
-                                  headers=self.headers)
-        model_id = resp.json['id']
-
-        definition = self.db.get_model_definition(model_id)
-        self.assertEquals(definition, MODEL_DEFINITION['definition'])
-
-    def test_cors_support_on_404(self):
-        headers = self.headers.copy()
-        headers['Origin'] = 'notmyidea.org'
-        response = self.app.get('/models/unknown/definition',
-                                headers=headers,
-                                status=404)
-        self.assertIn('Access-Control-Allow-Origin', response.headers)
-
     def test_permissions_unknown_retrieval(self):
         resp = self.app.get('/models/test/permissions',
                             headers=self.headers, status=404)
@@ -245,7 +308,7 @@ class ModelsViewsTest(BaseWebTest):
         )
         self.assertDictEqual(resp.json, permissions)
 
-    def test_patch_permissions_add(self):
+    def test_patch_permissions(self):
         self.app.put_json('/models/test',
                           MODEL_DEFINITION,
                           headers=self.headers)
@@ -263,101 +326,6 @@ class ModelsViewsTest(BaseWebTest):
         permissions[u"remy"] = [u"update_permissions"]
         self.assertDictEqual(resp.json, permissions)
 
-    def test_patch_permissions_remove(self):
-        self.app.put_json('/models/test',
-                          MODEL_DEFINITION,
-                          headers=self.headers)
-
-        resp = self.app.patch_json(
-            '/models/test/permissions',
-            {self.credentials['id']: ["-read_permissions",
-                                      "-update_permissions"]},
-            headers=self.headers)
-        permissions = force_unicode(
-            {self.credentials['id']: MODEL_PERMISSIONS}
-        )
-        permissions[self.credentials['id']].remove("read_permissions")
-        permissions[self.credentials['id']].remove("update_permissions")
-        self.assertDictEqual(resp.json, permissions)
-
-    def test_patch_permissions_add_all(self):
-        self.app.put_json('/models/test',
-                          MODEL_DEFINITION,
-                          headers=self.headers)
-        self.db.store_credentials('foo', {'id': 'alexis', 'key': 'bar'})
-
-        resp = self.app.patch_json('/models/test/permissions',
-                                   {"alexis": ["ALL"]},
-                                   headers=self.headers)
-        permissions = force_unicode(
-            {self.credentials['id']: MODEL_PERMISSIONS}
-        )
-        permissions[u"alexis"] = sorted(PERMISSIONS_SET)
-        self.assertDictEqual(resp.json, permissions)
-
-    def test_patch_permissions_add_system_principals(self):
-        self.app.put_json('/models/test',
-                          MODEL_DEFINITION,
-                          headers=self.headers)
-
-        resp = self.app.patch_json(
-            '/models/test/permissions',
-            {Everyone: ["read_definition"],
-             Authenticated: ["read_definition", "read_permissions"]},
-            headers=self.headers
-        )
-        permissions = force_unicode(
-            {self.credentials['id']: MODEL_PERMISSIONS}
-        )
-        permissions[Authenticated] = ["read_definition", "read_permissions"]
-        permissions[Everyone] = ["read_definition"]
-        self.assertDictEqual(resp.json, permissions)
-
-    def test_patch_permissions_add_shortcuts_principals(self):
-        self.app.put_json('/models/test',
-                          MODEL_DEFINITION,
-                          headers=self.headers)
-
-        resp = self.app.patch_json(
-            '/models/test/permissions',
-            {"Everyone": ["read_definition"],
-             "Authenticated": ["read_definition", "read_permissions"]},
-            headers=self.headers
-        )
-        permissions = force_unicode(
-            {self.credentials['id']: MODEL_PERMISSIONS}
-        )
-        permissions[Authenticated] = ["read_definition", "read_permissions"]
-        permissions[Everyone] = ["read_definition"]
-        self.assertDictEqual(resp.json, permissions)
-
-    def test_patch_permissions_remove_all(self):
-        self.app.put_json('/models/test',
-                          MODEL_DEFINITION,
-                          headers=self.headers)
-
-        resp = self.app.patch_json('/models/test/permissions',
-                                   {self.credentials['id']: ["-all"]},
-                                   headers=self.headers)
-        self.assertDictEqual(resp.json, {})
-
-    def test_patch_permissions_add_unknown(self):
-        self.app.put_json('/models/test',
-                          MODEL_DEFINITION,
-                          headers=self.headers)
-
-        resp = self.app.patch_json('/models/test/permissions',
-                                   {"alexis": ["read_permissions"]},
-                                   headers=self.headers,
-                                   status=400)
-        self.assertDictEqual(resp.json, force_unicode({
-            "status": "error",
-            "errors": [
-                {"location": "body", "name": "alexis",
-                 "description": "Credentials id couldn't be found."}
-            ]
-        }))
-
     def test_put_permissions(self):
         self.app.put_json('/models/test',
                           MODEL_DEFINITION,
@@ -374,52 +342,6 @@ class ModelsViewsTest(BaseWebTest):
         permissions["remy"] = ["update_permissions"]
         self.assertDictEqual(resp.json, force_unicode(permissions))
 
-    def test_put_permissions_wrong_identifier(self):
-        self.app.put_json('/models/test',
-                          MODEL_DEFINITION,
-                          headers=self.headers)
-
-        resp = self.app.put_json('/models/test/permissions',
-                                 {"alexis": ["read_permissions"]},
-                                 headers=self.headers,
-                                 status=400)
-
-        self.assertDictEqual(resp.json, force_unicode({
-            "status": "error",
-            "errors": [
-                {"location": "body", "name": "alexis",
-                 "description": "Credentials id couldn't be found."}
-            ]
-        }))
-
-    def test_put_permissions_wrong_perm(self):
-        self.app.put_json('/models/test',
-                          MODEL_DEFINITION,
-                          headers=self.headers)
-
-        resp = self.app.put_json('/models/test/permissions',
-                                 {"Everyone": ["foo", "read_permissions"]},
-                                 headers=self.headers,
-                                 status=400)
-
-        self.assertDictEqual(resp.json, force_unicode({
-            "status": "error",
-            "errors": [
-                {"location": "body", "name": "Everyone",
-                 "description": "Invalid permissions: foo"}
-            ]
-        }))
-
-    def test_definition_retrieval(self):
-        self.app.put_json('/models/test',
-                          MODEL_DEFINITION,
-                          headers=self.headers)
-
-        resp = self.app.get('/models/test/definition',
-                            headers=self.headers)
-        definition = force_unicode(MODEL_DEFINITION['definition'])
-        self.assertDictEqual(resp.json, definition)
-
     def test_post_model_definition_with_records(self):
         model = MODEL_DEFINITION.copy()
         model['records'] = [MODEL_RECORD, MODEL_RECORD]
@@ -428,19 +350,6 @@ class ModelsViewsTest(BaseWebTest):
         model_id = resp.json['id']
         self.assertEquals(len(self.db.get_records(model_id)), 2)
 
-    def test_put_model_definition_without_records(self):
-        model = MODEL_DEFINITION.copy()
-        model['records'] = [MODEL_RECORD, MODEL_RECORD]
-        resp = self.app.post_json('/models', model,
-                                  headers=self.headers)
-        model_id = resp.json['id']
-
-        model.pop('records')
-        resp = self.app.put_json('/models/%s' % model_id, model,
-                                 headers=self.headers)
-
-        self.assertEquals(len(self.db.get_records(model_id)), 0)
-
     def test_put_model_definition_with_records(self):
         model = MODEL_DEFINITION.copy()
         model['records'] = [MODEL_RECORD, MODEL_RECORD]
@@ -454,6 +363,31 @@ class ModelsViewsTest(BaseWebTest):
 
         self.assertEquals(len(self.db.get_records(model_id)), 1)
 
+    def test_post_model_definition_with_permissions(self):
+        model = MODEL_DEFINITION.copy()
+        model['permissions'] = {"Everyone": ["ALL"]}
+        resp = self.app.post_json('/models', model,
+                                  headers=self.headers)
+        model_id = resp.json['id']
+
+        definition = self.db.get_model_definition(model_id)
+        self.assertEquals(definition, MODEL_DEFINITION['definition'])
+        permissions = self.db.get_model_permissions(model_id)
+        self.assertEquals(invert_permissions_matrix(permissions),
+                          {self.credentials['id']: MODEL_PERMISSIONS,
+                           u'system.Everyone': MODEL_PERMISSIONS})
+
+    def test_put_model_definition_with_permissions(self):
+        model = MODEL_DEFINITION.copy()
+        model['permissions'] = {'Everyone': ['ALL']}
+        self.app.put_json('/models/test', model,
+                          headers=self.headers)
+
+        permissions = self.db.get_model_permissions("test")
+        self.assertEquals(invert_permissions_matrix(permissions),
+                          {self.credentials['id']: MODEL_PERMISSIONS,
+                           u'system.Everyone': MODEL_PERMISSIONS})
+
     def test_put_model_definition_new(self):
         model = MODEL_DEFINITION.copy()
         resp = self.app.put_json('/models/toto', model)
@@ -468,15 +402,6 @@ class ModelsViewsTest(BaseWebTest):
 
         resp = self.app.put_json('/models/toto', model, status=401)
 
-    def test_malformed_definition_creation(self):
-        definition_without_title = MODEL_DEFINITION['definition'].copy()
-        definition_without_title.pop('title')
-        resp = self.app.put_json('/models/test',
-                                 {'definition': definition_without_title},
-                                 headers=self.headers,
-                                 status=400)
-        self.assertIn('"name": "title"', resp.body.decode('utf-8'))
-
     def test_definition_creation_rejects_malformed_json(self):
         malformed_definition = '{"test":"toto", "titi": "tutu'
         resp = self.app.put('/models/test',
@@ -521,6 +446,19 @@ class ModelsViewsTest(BaseWebTest):
 
 class RecordsViewsTest(BaseWebTest):
 
+    def test_deserialized_value_is_stored(self):
+        definition = copy.deepcopy(MODEL_DEFINITION)
+        definition['definition']['fields'] = [{
+            'name': 'updated',
+            'type': 'date',
+            'autonow': True}]
+        self.app.put_json('/models/test', definition,
+                          headers=self.headers)
+        resp = self.app.post_json('/models/test/records', {},
+                                  headers=self.headers)
+        stored = self.db.get_record('test', resp.json['id'])
+        self.assertIsNotNone(stored.get('updated'))
+
     def test_delete_model_records(self):
         self.app.put_json('/models/test', MODEL_DEFINITION,
                           headers=self.headers)
@@ -656,6 +594,11 @@ class RecordsViewsTest(BaseWebTest):
 
 class CreateTokenViewTest(BaseWebTest):
 
+    def setUp(self):
+        super(CreateTokenViewTest, self).setUp()
+        userpass = u'foolish:bar'.encode('ascii')
+        self.auth = base64.b64encode(userpass).strip().decode('ascii')
+
     def test_post_token(self):
         response = self.app.post('/tokens', status=201)
         self.assertIn("token", response.json)
@@ -667,6 +610,36 @@ class CreateTokenViewTest(BaseWebTest):
         self.assertTrue(len(response.json["credentials"]["key"]) == 64)
         self.assertEqual("sha256", response.json["credentials"]["algorithm"])
 
+    def test_post_token_with_basic_auth(self):
+        response = self.app.post('/tokens', status=201, headers={
+            'Authorization': 'Basic {0}'.format(self.auth)
+        })
+        credentials = response.json
+        response = self.app.post('/tokens', status=200, headers={
+            'Authorization': 'Basic {0}'.format(self.auth)
+        })
+        self.assertEqual(credentials, response.json)
+
+    def test_post_token_with_token_authorization(self):
+        response = self.app.post('/tokens', status=201, headers={
+            'Authorization': 'Token {0}'.format(self.auth)
+        })
+        credentials = response.json
+        response = self.app.post('/tokens', status=200, headers={
+            'Authorization': 'Token {0}'.format(self.auth)
+        })
+        self.assertEqual(credentials, response.json)
+
+    def test_post_token_is_not_the_same_for_basic_or_token(self):
+        response = self.app.post('/tokens', status=201, headers={
+            'Authorization': 'Token {0}'.format(self.auth)
+        })
+        credentials = response.json
+        response = self.app.post('/tokens', status=201, headers={
+            'Authorization': 'Basic {0}'.format(self.auth)
+        })
+        self.assertNotEqual(credentials, response.json)
+
 
 class TokenViewTest(BaseWebTest):
 
@@ -697,6 +670,9 @@ class SearchViewTest(BaseWebTest):
                           headers=self.headers)
 
     def test_search_returns_200_if_query_is_correct(self):
+        self.app.post('/models/test/search/', {'match_all': {}},
+                      headers=self.headers,
+                      status=200)
         self.app.get('/models/test/search/', {'match_all': {}},
                      headers=self.headers,
                      status=200)
@@ -705,9 +681,9 @@ class SearchViewTest(BaseWebTest):
     def test_search_supports_query_string_parameters(self, search_mock):
         search_mock.return_value = {}
         query = {'match_all': {}}
-        self.app.get('/models/test/search/?size=100', query,
-                     headers=self.headers,
-                     status=200)
+        self.app.post('/models/test/search/?size=100', query,
+                      headers=self.headers,
+                      status=200)
         search_mock.called_with(index='test', doc_type='test',
                                 body=query, size=100)
 
@@ -733,6 +709,15 @@ class SearchViewTest(BaseWebTest):
                      headers=self.headers,
                      status=502)
 
+    @mock.patch('elasticsearch.client.Elasticsearch.search')
+    def test_search_returns_original_code_on_bad_request(self, search_mock):
+        badrequest = elasticsearch.RequestError('400', 'error', {'foo': 'bar'})
+        search_mock.side_effect = badrequest
+        resp = self.app.get('/models/test/search/', {},
+                            headers=self.headers,
+                            status=400)
+        self.assertEqual(resp.json['msg']['foo'], 'bar')
+
     def test_search_view_requires_permission(self):
         self.app.patch_json('/models/test/permissions',
                             {self.credentials['id']: ["-read_all_records"]},
@@ -740,3 +725,61 @@ class SearchViewTest(BaseWebTest):
         self.app.get('/models/test/search/', {},
                      headers=self.headers,
                      status=403)
+
+
+class CORSHeadersTest(BaseWebTest):
+
+    def setUp(self):
+        super(CORSHeadersTest, self).setUp()
+        self.headers['Origin'] = 'notmyidea.org'
+
+    def test_support_on_options_404(self):
+        headers = self.headers.copy()
+        headers['Access-Control-Request-Method'] = 'GET'
+        response = self.app.options('/models/unknown/definition',
+                                    headers=headers,
+                                    status=200)
+        self.assertIn('Access-Control-Allow-Origin', response.headers)
+
+    def test_support_on_get_unknown_model(self):
+        response = self.app.get('/models/unknown/definition',
+                                headers=self.headers,
+                                status=404)
+        self.assertIn('Access-Control-Allow-Origin', response.headers)
+
+    def test_support_on_valid_definition(self):
+        response = self.app.put_json('/models/test',
+                                     MODEL_DEFINITION,
+                                     headers=self.headers,
+                                     status=200)
+        self.assertIn('Access-Control-Allow-Origin', response.headers)
+
+    def test_support_on_invalid_definition(self):
+        definition = copy.deepcopy(MODEL_DEFINITION)
+        definition['definition'].pop('fields')
+        response = self.app.put_json('/models/test',
+                                     definition,
+                                     headers=self.headers,
+                                     status=400)
+        self.assertIn('Access-Control-Allow-Origin', response.headers)
+
+    def test_support_on_unauthorized(self):
+        response = self.app.get('/token',
+                                MODEL_DEFINITION,
+                                headers={'Origin': 'notmyidea.org'},
+                                status=401)
+        self.assertIn('Access-Control-Allow-Origin', response.headers)
+
+    def test_support_on_forbidden(self):
+        self.app.put_json('/models/test',
+                          MODEL_DEFINITION,
+                          headers=self.headers,
+                          status=200)
+        self.app.patch_json('/models/test/permissions',
+                            {self.credentials['id']: ["-ALL"]},
+                            headers=self.headers,
+                            status=200)
+        response = self.app.get('/models/test',
+                                headers=self.headers,
+                                status=403)
+        self.assertIn('Access-Control-Allow-Origin', response.headers)
